@@ -1,24 +1,10 @@
-# (C) British Crown Copyright 2014 - 2019, Met Office
+# Copyright Iris contributors
 #
-# This file is part of Iris.
-#
-# Iris is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Iris is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with Iris.  If not, see <http://www.gnu.org/licenses/>.
+# This file is part of Iris and is released under the LGPL license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
 """A collection of helpers for interpolation."""
 
-from __future__ import (absolute_import, division, print_function)
-from six.moves import (filter, input, map, range, zip)  # noqa
-
 from collections import namedtuple
 from itertools import product
 import operator
@@ -36,17 +22,17 @@ import iris.util
 _DEFAULT_DTYPE = np.float16
 
 
-ExtrapolationMode = namedtuple('ExtrapolationMode', ['bounds_error',
-                                                     'fill_value',
-                                                     'mask_fill_value',
-                                                     'force_mask'])
+ExtrapolationMode = namedtuple(
+    "ExtrapolationMode",
+    ["bounds_error", "fill_value", "mask_fill_value", "force_mask"],
+)
 
 EXTRAPOLATION_MODES = {
-    'extrapolate': ExtrapolationMode(False, None, None, False),
-    'error': ExtrapolationMode(True, 0, 0, False),
-    'nan': ExtrapolationMode(False, np.nan, 0, False),
-    'mask': ExtrapolationMode(False, np.nan, 1, True),
-    'nanmask': ExtrapolationMode(False, np.nan, 1, False)
+    "extrapolate": ExtrapolationMode(False, None, None, False),
+    "error": ExtrapolationMode(True, 0, 0, False),
+    "nan": ExtrapolationMode(False, np.nan, 0, False),
+    "mask": ExtrapolationMode(False, np.nan, 1, True),
+    "nanmask": ExtrapolationMode(False, np.nan, 1, False),
 }
 
 
@@ -61,12 +47,14 @@ def _canonical_sample_points(coords, sample_points):
     canonical_sample_points = []
     for coord, points in zip(coords, sample_points):
         if coord.units.is_time_reference():
+
             def convert_date(date):
                 try:
                     date = coord.units.date2num(date)
                 except AttributeError:
                     pass
                 return date
+
             convert_dates = np.vectorize(convert_date, [np.dtype(float)])
             points = convert_dates(points)
         canonical_sample_points.append(points)
@@ -79,8 +67,7 @@ def extend_circular_coord(coord, points):
     This is common when dealing with circular coordinates.
 
     """
-    modulus = np.array(coord.units.modulus or 0,
-                       dtype=coord.dtype)
+    modulus = np.array(coord.units.modulus or 0, dtype=coord.dtype)
     points = np.append(points, points[0] + modulus)
     return points
 
@@ -102,9 +89,9 @@ def extend_circular_data(data, coord_dim):
     coord_slice_in_cube[coord_dim] = slice(0, 1)
 
     mod = ma if ma.isMaskedArray(data) else np
-    data = mod.concatenate((data,
-                            data[tuple(coord_slice_in_cube)]),
-                           axis=coord_dim)
+    data = mod.concatenate(
+        (data, data[tuple(coord_slice_in_cube)]), axis=coord_dim
+    )
     return data
 
 
@@ -153,22 +140,28 @@ def get_xy_coords(cube, dim_coords=False):
         A tuple containing the cube's x and y dimension coordinates.
 
     """
-    x_coords = cube.coords(axis='x', dim_coords=dim_coords)
+    x_coords = cube.coords(axis="x", dim_coords=dim_coords)
     if len(x_coords) != 1 or x_coords[0].ndim != 1:
-        raise ValueError('Cube {!r} must contain a single 1D x '
-                         'coordinate.'.format(cube.name()))
+        raise ValueError(
+            "Cube {!r} must contain a single 1D x "
+            "coordinate.".format(cube.name())
+        )
     x_coord = x_coords[0]
 
-    y_coords = cube.coords(axis='y', dim_coords=dim_coords)
+    y_coords = cube.coords(axis="y", dim_coords=dim_coords)
     if len(y_coords) != 1 or y_coords[0].ndim != 1:
-        raise ValueError('Cube {!r} must contain a single 1D y '
-                         'coordinate.'.format(cube.name()))
+        raise ValueError(
+            "Cube {!r} must contain a single 1D y "
+            "coordinate.".format(cube.name())
+        )
     y_coord = y_coords[0]
 
     if x_coord.coord_system != y_coord.coord_system:
-        raise ValueError("The cube's x ({!r}) and y ({!r}) "
-                         "coordinates must have the same coordinate "
-                         "system.".format(x_coord.name(), y_coord.name()))
+        raise ValueError(
+            "The cube's x ({!r}) and y ({!r}) "
+            "coordinates must have the same coordinate "
+            "system.".format(x_coord.name(), y_coord.name())
+        )
 
     return x_coord, y_coord
 
@@ -183,12 +176,13 @@ def snapshot_grid(cube):
     return x.copy(), y.copy()
 
 
-class RectilinearInterpolator(object):
+class RectilinearInterpolator:
     """
     This class provides support for performing nearest-neighbour or
     linear interpolation over one or more orthogonal dimensions.
 
     """
+
     def __init__(self, src_cube, coords, method, extrapolation_mode):
         """
         Perform interpolation over one or more orthogonal coordinates.
@@ -230,13 +224,13 @@ class RectilinearInterpolator(object):
         # Coordinates defining the dimensions to be interpolated.
         self._src_coords = [self._src_cube.coord(coord) for coord in coords]
         # Whether to use linear or nearest-neighbour interpolation.
-        if method not in ('linear', 'nearest'):
-            msg = 'Interpolation method {!r} not supported'.format(method)
+        if method not in ("linear", "nearest"):
+            msg = "Interpolation method {!r} not supported".format(method)
             raise ValueError(msg)
         self._method = method
         # The extrapolation mode.
         if extrapolation_mode not in EXTRAPOLATION_MODES:
-            msg = 'Extrapolation mode {!r} not supported.'
+            msg = "Extrapolation mode {!r} not supported."
             raise ValueError(msg.format(extrapolation_mode))
         self._mode = extrapolation_mode
         # The point values defining the dimensions to be interpolated.
@@ -280,8 +274,9 @@ class RectilinearInterpolator(object):
                 # Map all the requested values into the range of the source
                 # data (centred over the centre of the source data to allow
                 # extrapolation where required).
-                points[:, index] = wrap_circular_points(points[:, index],
-                                                        offset, modulus)
+                points[:, index] = wrap_circular_points(
+                    points[:, index], offset, modulus
+                )
 
             # Also extend data if circular (to match the coord points, which
             # 'setup' already extended).
@@ -293,8 +288,9 @@ class RectilinearInterpolator(object):
     def _account_for_inverted(self, data):
         if np.any(self._coord_decreasing):
             dim_slices = [slice(None)] * data.ndim
-            for interp_dim, flip in zip(self._interp_dims,
-                                        self._coord_decreasing):
+            for interp_dim, flip in zip(
+                self._interp_dims, self._coord_decreasing
+            ):
                 if flip:
                     dim_slices[interp_dim] = slice(-1, None, -1)
             data = data[tuple(dim_slices)]
@@ -341,8 +337,12 @@ class RectilinearInterpolator(object):
             # some unnecessary checks on the fill_value parameter,
             # so we set it afterwards instead. Sneaky. ;-)
             self._interpolator = _RegularGridInterpolator(
-                self._src_points, data, method=self.method,
-                bounds_error=mode.bounds_error, fill_value=None)
+                self._src_points,
+                data,
+                method=self.method,
+                bounds_error=mode.bounds_error,
+                fill_value=None,
+            )
         else:
             self._interpolator.values = data
 
@@ -365,7 +365,7 @@ class RectilinearInterpolator(object):
             self._interpolator.fill_value = mode.mask_fill_value
             self._interpolator.values = src_mask
             mask_fraction = self._interpolator(interp_points)
-            new_mask = (mask_fraction > 0)
+            new_mask = mask_fraction > 0
             if ma.isMaskedArray(data) or np.any(new_mask):
                 result = np.ma.MaskedArray(result, new_mask)
 
@@ -384,8 +384,10 @@ class RectilinearInterpolator(object):
         # - By expanding to N dimensions self._points() is doing
         #   unnecessary work.
         data = self._points(sample_points, coord.points, coord_dims)
-        index = tuple(0 if dim not in coord_dims else slice(None)
-                      for dim in range(self._src_cube.ndim))
+        index = tuple(
+            0 if dim not in coord_dims else slice(None)
+            for dim in range(self._src_cube.ndim)
+        )
         new_points = data[index]
         # Watch out for DimCoord instances that are no longer monotonic
         # after the resampling.
@@ -416,33 +418,36 @@ class RectilinearInterpolator(object):
             #    Force all coordinates to be monotonically increasing.
             #    Generally this isn't always necessary for a rectilinear
             #    interpolator, but it is a common requirement.)
-            decreasing = (coord.ndim == 1 and
-                          # NOTE: this clause avoids an error when > 1D,
-                          # as '_validate' raises a more readable error.
-                          coord_points.size > 1 and
-                          coord_points[1] < coord_points[0])
+            decreasing = (
+                coord.ndim == 1
+                and
+                # NOTE: this clause avoids an error when > 1D,
+                # as '_validate' raises a more readable error.
+                coord_points.size > 1
+                and coord_points[1] < coord_points[0]
+            )
             self._coord_decreasing.append(decreasing)
             if decreasing:
                 coord_points = coord_points[::-1]
 
             # Record info if coord is circular, and adjust points.
-            circular = getattr(coord, 'circular', False)
-            modulus = getattr(coord.units, 'modulus', 0)
+            circular = getattr(coord, "circular", False)
+            modulus = getattr(coord.units, "modulus", 0)
             if circular or modulus:
                 # Only DimCoords can be circular.
                 if circular:
                     coord_points = extend_circular_coord(coord, coord_points)
-                offset = 0.5 * (coord_points.max() + coord_points.min() -
-                                modulus)
-                self._circulars.append((circular, modulus,
-                                        index, coord_dims[0],
-                                        offset))
+                offset = 0.5 * (
+                    coord_points.max() + coord_points.min() - modulus
+                )
+                self._circulars.append(
+                    (circular, modulus, index, coord_dims[0], offset)
+                )
 
             self._src_points.append(coord_points)
 
             # Record any interpolation cube dims we haven't already seen.
-            coord_dims = [c for c in coord_dims
-                          if c not in self._interp_dims]
+            coord_dims = [c for c in coord_dims if c not in self._interp_dims]
             self._interp_dims += coord_dims
 
         self._validate()
@@ -455,19 +460,25 @@ class RectilinearInterpolator(object):
 
         """
         if len(set(self._interp_dims)) != len(self._src_coords):
-            raise ValueError('Coordinates repeat a data dimension - the '
-                             'interpolation would be over-specified.')
+            raise ValueError(
+                "Coordinates repeat a data dimension - the "
+                "interpolation would be over-specified."
+            )
 
         for coord in self._src_coords:
             if coord.ndim != 1:
-                raise ValueError('Interpolation coords must be 1-d for '
-                                 'rectilinear interpolation.')
+                raise ValueError(
+                    "Interpolation coords must be 1-d for "
+                    "rectilinear interpolation."
+                )
 
             if not isinstance(coord, DimCoord):
                 # Check monotonic.
                 if not iris.util.monotonic(coord.points, strict=True):
-                    msg = 'Cannot interpolate over the non-' \
-                        'monotonic coordinate {}.'
+                    msg = (
+                        "Cannot interpolate over the non-"
+                        "monotonic coordinate {}."
+                    )
                     raise ValueError(msg.format(coord.name()))
 
     def _interpolated_dtype(self, dtype):
@@ -476,7 +487,7 @@ class RectilinearInterpolator(object):
         underlying interpolator.
 
         """
-        if self._method == 'nearest':
+        if self._method == "nearest":
             result = dtype
         else:
             result = np.result_type(_DEFAULT_DTYPE, dtype)
@@ -515,13 +526,15 @@ class RectilinearInterpolator(object):
         data_dims = data_dims or dims
 
         if len(data_dims) != data.ndim:
-            msg = 'Data being interpolated is not consistent with ' \
-                'the data passed through.'
+            msg = (
+                "Data being interpolated is not consistent with "
+                "the data passed through."
+            )
             raise ValueError(msg)
 
         if sorted(data_dims) != list(data_dims):
             # To do this, a pre & post transpose will be necessary.
-            msg = 'Currently only increasing data_dims is supported.'
+            msg = "Currently only increasing data_dims is supported."
             raise NotImplementedError(msg)
 
         # Broadcast the data into the shape of the original cube.
@@ -530,8 +543,9 @@ class RectilinearInterpolator(object):
             for dim in range(self._src_cube.ndim):
                 if dim not in data_dims:
                     strides.insert(dim, 0)
-            data = as_strided(data, strides=strides,
-                              shape=self._src_cube.shape)
+            data = as_strided(
+                data, strides=strides, shape=self._src_cube.shape
+            )
 
         data = self._account_for_inverted(data)
         # Calculate the transpose order to shuffle the interpolated dimensions
@@ -541,8 +555,9 @@ class RectilinearInterpolator(object):
         di = self._interp_dims
         ds = sorted(dims, key=lambda d: d not in di)
         dmap = {d: di.index(d) if d in di else ds.index(d) for d in dims}
-        interp_order, _ = zip(*sorted(dmap.items(),
-                                      key=operator.itemgetter(1)))
+        interp_order, _ = zip(
+            *sorted(dmap.items(), key=operator.itemgetter(1))
+        )
         _, src_order = zip(*sorted(dmap.items(), key=operator.itemgetter(0)))
 
         # Prepare the sample points for interpolation and calculate the
@@ -555,8 +570,9 @@ class RectilinearInterpolator(object):
             interp_points.append(points)
             interp_shape.append(points.size)
 
-        interp_shape.extend(length for dim, length in enumerate(data.shape) if
-                            dim not in di)
+        interp_shape.extend(
+            length for dim, length in enumerate(data.shape) if dim not in di
+        )
 
         # Convert the interpolation points into a cross-product array
         # with shape (n_cross_points, n_dims)
@@ -604,12 +620,14 @@ class RectilinearInterpolator(object):
 
         """
         if len(sample_points) != len(self._src_coords):
-            msg = 'Expected sample points for {} coordinates, got {}.'
-            raise ValueError(msg.format(len(self._src_coords),
-                                        len(sample_points)))
+            msg = "Expected sample points for {} coordinates, got {}."
+            raise ValueError(
+                msg.format(len(self._src_coords), len(sample_points))
+            )
 
-        sample_points = _canonical_sample_points(self._src_coords,
-                                                 sample_points)
+        sample_points = _canonical_sample_points(
+            self._src_coords, sample_points
+        )
 
         data = self._src_cube.data
         # Interpolate the cube payload.
@@ -654,15 +672,19 @@ class RectilinearInterpolator(object):
             else:
                 if set(dims).intersection(set(self._interp_dims)):
                     # Interpolate the coordinate payload.
-                    new_coord = self._resample_coord(sample_points, coord,
-                                                     dims)
+                    new_coord = self._resample_coord(
+                        sample_points, coord, dims
+                    )
                 else:
                     new_coord = coord.copy()
             return new_coord, dims
 
         def gen_new_cube():
-            if (isinstance(new_coord, DimCoord) and len(dims) > 0 and
-                    dims[0] not in dims_with_dim_coords):
+            if (
+                isinstance(new_coord, DimCoord)
+                and len(dims) > 0
+                and dims[0] not in dims_with_dim_coords
+            ):
                 new_cube._add_unique_dim_coord(new_coord, dims)
                 dims_with_dim_coords.append(dims[0])
             else:
@@ -670,7 +692,7 @@ class RectilinearInterpolator(object):
             coord_mapping[id(coord)] = new_coord
 
         # Copy/interpolate the coordinates.
-        for coord in (cube.dim_coords + cube.aux_coords):
+        for coord in cube.dim_coords + cube.aux_coords:
             new_coord, dims = construct_new_coord(coord)
             gen_new_cube()
 
@@ -678,8 +700,10 @@ class RectilinearInterpolator(object):
             new_cube.add_aux_factory(factory.updated(coord_mapping))
 
         if collapse_scalar and _new_scalar_dims:
-            dim_slices = [0 if dim in _new_scalar_dims else slice(None)
-                          for dim in range(new_cube.ndim)]
+            dim_slices = [
+                0 if dim in _new_scalar_dims else slice(None)
+                for dim in range(new_cube.ndim)
+            ]
             new_cube = new_cube[tuple(dim_slices)]
 
         return new_cube
